@@ -8,7 +8,7 @@
 # or in the "LICENSE.txt" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES
 # OR CONDITIONS OF ANY KIND, express or implied. See the License for the specific language governing permissions and
 # limitations under the License.
-from future.moves.collections import OrderedDict
+from collections import OrderedDict
 
 import boto3
 
@@ -21,14 +21,19 @@ from pcluster.config.cfn_param_types import (
     ClusterCfnSection,
     ClusterConfigMetadataCfnParam,
     ComputeAvailabilityZoneCfnParam,
+    ComputeInstanceTypeCfnParam,
     DisableHyperThreadingCfnParam,
     EBSSettingsCfnParam,
     EFSCfnSection,
     ExtraJsonCfnParam,
     FloatCfnParam,
+    FSxDNSNameParam,
+    FSxMountNameParam,
+    HeadNodeAvailabilityZoneCfnParam,
+    HeadNodeInstanceTypeCfnParam,
     IntCfnParam,
+    JsonCfnParam,
     MaintainInitialSizeCfnParam,
-    MasterAvailabilityZoneCfnParam,
     NetworkInterfacesCountCfnParam,
     QueueSizeCfnParam,
     SettingsCfnParam,
@@ -36,6 +41,7 @@ from pcluster.config.cfn_param_types import (
     SpotBidPercentageCfnParam,
     SpotPriceCfnParam,
     TagsParam,
+    VolumeIopsParam,
     VolumeSizeParam,
 )
 from pcluster.config.json_param_types import (
@@ -53,7 +59,7 @@ from pcluster.config.param_types import Visibility
 from pcluster.config.update_policy import UpdatePolicy
 from pcluster.config.validators import (
     architecture_os_validator,
-    base_os_validator,
+    cluster_type_validator,
     cluster_validator,
     compute_instance_type_validator,
     compute_resource_validator,
@@ -64,6 +70,7 @@ from pcluster.config.validators import (
     ebs_settings_validator,
     ebs_volume_iops_validator,
     ebs_volume_size_snapshot_validator,
+    ebs_volume_throughput_validator,
     ebs_volume_type_size_validator,
     ec2_ami_validator,
     ec2_iam_policies_validator,
@@ -75,6 +82,7 @@ from pcluster.config.validators import (
     ec2_volume_validator,
     ec2_vpc_id_validator,
     efa_gdr_validator,
+    efa_os_arch_validator,
     efa_validator,
     efs_id_validator,
     efs_validator,
@@ -93,9 +101,11 @@ from pcluster.config.validators import (
     intel_hpc_os_validator,
     kms_key_validator,
     maintain_initial_size_validator,
+    queue_compute_type_validator,
     queue_settings_validator,
     queue_validator,
-    raid_volume_iops_validator,
+    region_validator,
+    s3_bucket_region_validator,
     s3_bucket_uri_validator,
     s3_bucket_validator,
     scheduler_validator,
@@ -103,9 +113,15 @@ from pcluster.config.validators import (
     tags_validator,
     url_validator,
 )
-from pcluster.constants import CIDR_ALL_IPS, FSX_HDD_THROUGHPUT, FSX_SSD_THROUGHPUT, SUPPORTED_ARCHITECTURES
+from pcluster.constants import (
+    CIDR_ALL_IPS,
+    FSX_HDD_THROUGHPUT,
+    FSX_SSD_THROUGHPUT,
+    SUPPORTED_ARCHITECTURES,
+    SUPPORTED_OSS,
+)
 
-CLUSTER_COMMON_VALIDATORS = [duplicate_shared_dir_validator]
+CLUSTER_COMMON_VALIDATORS = [duplicate_shared_dir_validator, region_validator]
 # This file contains a definition of all the sections and the parameters configurable by the user
 # in the configuration file.
 
@@ -144,12 +160,12 @@ ALLOWED_VALUES = {
     "efs_fs_id": r"^fs-[0-9a-z]{8}$|^fs-[0-9a-z]{17}|NONE$",
     "file_path": r"^\/?[^\/.\\][^\/\\]*(\/[^\/.\\][^\/]*)*$",
     "fsx_fs_id": r"^fs-[0-9a-z]{17}|NONE$",
-    "greater_than_25": r"^([0-9]+[0-9]{2}|[3-9][0-9]|2[5-9])$",
+    "greater_than_35": r"^([1-9]+[0-9]{2}|[4-9][0-9]|3[5-9])$",
     "security_group_id": r"^sg-[0-9a-z]{8}$|^sg-[0-9a-z]{17}$",
     "snapshot_id": r"^snap-[0-9a-z]{8}$|^snap-[0-9a-z]{17}$",
     "subnet_id": r"^subnet-[0-9a-z]{8}$|^subnet-[0-9a-z]{17}$",
     "volume_id": r"^vol-[0-9a-z]{8}$|^vol-[0-9a-z]{17}$",
-    "volume_types": ["standard", "io1", "gp2", "st1", "sc1"],
+    "volume_types": ["standard", "io1", "io2", "gp2", "st1", "sc1", "gp3"],
     "vpc_id": r"^vpc-[0-9a-z]{8}$|^vpc-[0-9a-z]{17}$",
     "fsx_deployment_type": ["SCRATCH_1", "SCRATCH_2", "PERSISTENT_1"],
     "fsx_ssd_throughput": FSX_SSD_THROUGHPUT,
@@ -157,7 +173,8 @@ ALLOWED_VALUES = {
     "architectures": SUPPORTED_ARCHITECTURES,
     "fsx_auto_import_policy": ["NEW", "NEW_CHANGED"],
     "fsx_storage_type": ["SSD", "HDD"],
-    "fsx_drive_cache_type": ["READ"]
+    "fsx_drive_cache_type": ["READ"],
+    "fsx_data_compression_type": ["LZ4"]
 }
 
 AWS = {
@@ -239,68 +256,70 @@ VPC = {
     "key": "vpc",
     "default_label": "default",
     "autocreate": True,
-    "params": {
-        "vpc_id": {
+    "params": OrderedDict([
+        ("vpc_id", {
             "cfn_param_mapping": "VPCId",
+            "required": True,
             "allowed_values": ALLOWED_VALUES["vpc_id"],
             "validators": [ec2_vpc_id_validator],
             "update_policy": UpdatePolicy.UNSUPPORTED
-        },
-        "master_subnet_id": {
+        }),
+        ("master_subnet_id", {
             "cfn_param_mapping": "MasterSubnetId",
+            "required": True,
             "allowed_values": ALLOWED_VALUES["subnet_id"],
             "validators": [ec2_subnet_id_validator],
             "update_policy": UpdatePolicy.UNSUPPORTED
-        },
-        "ssh_from": {
+        }),
+        ("ssh_from", {
             "default": CIDR_ALL_IPS,
             "allowed_values": ALLOWED_VALUES["cidr"],
             "cfn_param_mapping": "AccessFrom",
             "update_policy": UpdatePolicy.SUPPORTED
-        },
-        "additional_sg": {
+        }),
+        ("additional_sg", {
             "cfn_param_mapping": "AdditionalSG",
             "allowed_values": ALLOWED_VALUES["security_group_id"],
             "validators": [ec2_security_group_validator],
             "update_policy": UpdatePolicy.SUPPORTED
-        },
-        "compute_subnet_id": {
+        }),
+        ("compute_subnet_id", {
             "cfn_param_mapping": "ComputeSubnetId",
             "allowed_values": ALLOWED_VALUES["subnet_id"],
             "validators": [ec2_subnet_id_validator],
             "update_policy": UpdatePolicy.COMPUTE_FLEET_STOP
-        },
-        "compute_subnet_cidr": {
+        }),
+        ("compute_subnet_cidr", {
             "cfn_param_mapping": "ComputeSubnetCidr",
             "allowed_values": ALLOWED_VALUES["cidr"],
             "update_policy": UpdatePolicy.UNSUPPORTED
-        },
-        "use_public_ips": {
+        }),
+        ("use_public_ips", {
             "type": BoolCfnParam,
             "default": True,
             "cfn_param_mapping": "UsePublicIps",
             "update_policy": UpdatePolicy.COMPUTE_FLEET_STOP
-        },
-        "vpc_security_group_id": {
+        }),
+        ("vpc_security_group_id", {
             "cfn_param_mapping": "VPCSecurityGroupId",
             "allowed_values": ALLOWED_VALUES["security_group_id"],
             "validators": [ec2_security_group_validator],
             "update_policy": UpdatePolicy.SUPPORTED
-        },
-        "master_availability_zone": {
+        }),
+        ("master_availability_zone", {
             # NOTE: this is not exposed as a configuration parameter
-            "type": MasterAvailabilityZoneCfnParam,
+            "type": HeadNodeAvailabilityZoneCfnParam,
             "cfn_param_mapping": "AvailabilityZone",
             "update_policy": UpdatePolicy.IGNORED,
             "visibility": Visibility.PRIVATE
-        },
-        "compute_availability_zone": {
+        }),
+        ("compute_availability_zone", {
             # NOTE: this is not exposed as a configuration parameter
             "type": ComputeAvailabilityZoneCfnParam,
             "update_policy": UpdatePolicy.IGNORED,
             "visibility": Visibility.PRIVATE
-        }
-    },
+        })
+    ]),
 }
 
 EBS = {
@@ -308,20 +327,21 @@ EBS = {
     "key": "ebs",
     "default_label": "default",
     "max_resources": 5,
-    "validators": [ebs_volume_type_size_validator, ebs_volume_iops_validator, ebs_volume_size_snapshot_validator],
-    "params": {
-        "shared_dir": {
+    "validators": [ebs_volume_type_size_validator, ebs_volume_iops_validator, ebs_volume_size_snapshot_validator,
+                   ebs_volume_throughput_validator],
+    "params": OrderedDict([
+        ("shared_dir", {
             "allowed_values": ALLOWED_VALUES["file_path"],
             "cfn_param_mapping": "SharedDir",
             "validators": [shared_dir_validator],
             "update_policy": UpdatePolicy.UNSUPPORTED
-        },
-        "ebs_snapshot_id": {
+        }),
+        ("ebs_snapshot_id", {
             "allowed_values": ALLOWED_VALUES["snapshot_id"],
             "cfn_param_mapping": "EBSSnapshotId",
             "update_policy": UpdatePolicy.UNSUPPORTED
-        },
-        "volume_type": {
+        }),
+        ("volume_type", {
             "default": "gp2",
             "allowed_values": ALLOWED_VALUES["volume_types"],
             "cfn_param_mapping": "VolumeType",
@@ -329,8 +349,8 @@ EBS = {
                 UpdatePolicy.UNSUPPORTED,
                 action_needed=UpdatePolicy.ACTIONS_NEEDED["ebs_volume_update"]
             )
-        },
-        "volume_size": {
+        }),
+        ("volume_size", {
             "type": VolumeSizeParam,
             "cfn_param_mapping": "VolumeSize",
             "update_policy": UpdatePolicy(
@@ -338,31 +358,36 @@ EBS = {
                 fail_reason=UpdatePolicy.FAIL_REASONS["ebs_volume_resize"],
                 action_needed=UpdatePolicy.ACTIONS_NEEDED["ebs_volume_update"]
             )
-        },
-        "volume_iops": {
-            "type": IntCfnParam,
-            "default": 100,
+        }),
+        ("volume_iops", {
+            "type": VolumeIopsParam,
             "cfn_param_mapping": "VolumeIOPS",
             "update_policy": UpdatePolicy.SUPPORTED
-        },
-        "encrypted": {
+        }),
+        ("encrypted", {
             "type": BoolCfnParam,
             "cfn_param_mapping": "EBSEncryption",
             "default": False,
             "update_policy": UpdatePolicy.UNSUPPORTED
-        },
-        "ebs_kms_key_id": {
+        }),
+        ("ebs_kms_key_id", {
             "cfn_param_mapping": "EBSKMSKeyId",
             "validators": [kms_key_validator],
             "update_policy": UpdatePolicy.UNSUPPORTED
-        },
-        "ebs_volume_id": {
+        }),
+        ("ebs_volume_id", {
             "cfn_param_mapping": "EBSVolumeId",
             "allowed_values": ALLOWED_VALUES["volume_id"],
             "validators": [ec2_volume_validator],
             "update_policy": UpdatePolicy.UNSUPPORTED
-        },
-    },
+        }),
+        ("volume_throughput", {
+            "type": IntCfnParam,
+            "cfn_param_mapping": "VolumeThroughput",
+            "update_policy": UpdatePolicy.SUPPORTED,
+            "default": 125
+        })
+    ]),
 }
 
 EFS = {
@@ -415,6 +440,7 @@ RAID = {
     "type": CfnSection,
     "key": "raid",
     "default_label": "default",
+    "validators": [ebs_volume_type_size_validator, ebs_volume_iops_validator, ebs_volume_throughput_validator],
     "cfn_param_mapping": "RAIDOptions",  # All the parameters in the section are converted into a single CFN parameter
     "params": OrderedDict(  # Use OrderedDict because the parameters must respect the order in the CFN parameter
         [
@@ -446,9 +472,7 @@ RAID = {
                 "update_policy": UpdatePolicy.UNSUPPORTED
             }),
             ("volume_iops", {
-                "type": IntCfnParam,
-                "default": 100,
-                "validators": [raid_volume_iops_validator],
+                "type": VolumeIopsParam,
                 "update_policy": UpdatePolicy.SUPPORTED
             }),
             ("encrypted", {
@@ -460,6 +484,12 @@ RAID = {
                 "validators": [kms_key_validator],
                 "update_policy": UpdatePolicy.UNSUPPORTED
             }),
+            ("volume_throughput", {
+                "type": IntCfnParam,
+                "default": 125,
+                "cfn_param_mapping": "VolumeThroughput",
+                "update_policy": UpdatePolicy.SUPPORTED
+            }),
         ]
     )
 }
@@ -548,7 +578,24 @@ FSX = {
                 "default": "NONE",
                 "allowed_values": ALLOWED_VALUES["fsx_drive_cache_type"],
                 "update_policy": UpdatePolicy.UNSUPPORTED
-            })
+            }),
+            ("existing_mount_name", {
+                "type": FSxMountNameParam,
+                "default": "NONE",
+                "update_policy": UpdatePolicy.IGNORED,
+                "visibility": Visibility.PRIVATE,
+            }),
+            ("existing_dns_name", {
+                "type": FSxDNSNameParam,
+                "default": "NONE",
+                "update_policy": UpdatePolicy.IGNORED,
+                "visibility": Visibility.PRIVATE,
+            }),
+            ("data_compression_type", {
+                "default": "NONE",
+                "allowed_values": ALLOWED_VALUES["fsx_data_compression_type"],
+                "update_policy": UpdatePolicy.SUPPORTED
+            }),
         ]
     )
 }
@@ -662,6 +709,13 @@ COMPUTE_RESOURCE = {
             "visibility": Visibility.PRIVATE,
             "default": 0
         }),
+        ("gpu_type", {
+            "type": JsonParam,
+            # This param is managed automatically
+            "update_policy": UpdatePolicy.IGNORED,
+            "visibility": Visibility.PRIVATE,
+            "default": "no_gpu_type"
+        }),
         ("network_interfaces", {
             "type": IntJsonParam,
             # This param is managed automatically
@@ -704,7 +758,7 @@ QUEUE = {
     "type": QueueJsonSection,
     "key": "queue",
     "default_label": "default",
-    "validators": [queue_validator],
+    "validators": [queue_validator, queue_compute_type_validator],
     "max_resources": 5,
     "params": OrderedDict([
         ("compute_type", {
@@ -715,6 +769,7 @@ QUEUE = {
         }),
         ("enable_efa", {
             "type": BooleanJsonParam,
+            "validators": [efa_os_arch_validator],
             "update_policy": UpdatePolicy.COMPUTE_FLEET_STOP,
         }),
         ("enable_efa_gdr", {
@@ -750,18 +805,9 @@ CLUSTER_COMMON_PARAMS = [
     }),
     ("key_name", {
         "cfn_param_mapping": "KeyName",
-        "required": True,
         "validators": [ec2_key_pair_validator],
         "update_policy": UpdatePolicy.UNSUPPORTED
     }),
-    ("base_os", {
-        "type": BaseOSCfnParam,
-        "cfn_param_mapping": "BaseOS",
-        "allowed_values": ["alinux", "alinux2", "ubuntu1604", "ubuntu1804", "centos7", "centos8"],
-        "validators": [base_os_validator, architecture_os_validator],
-        "required": True,
-        "update_policy": UpdatePolicy.UNSUPPORTED
-    }),
     ("scheduler", {
         "cfn_param_mapping": "Scheduler",
         "allowed_values": ["awsbatch", "sge", "slurm", "torque"],
@@ -769,17 +815,17 @@ CLUSTER_COMMON_PARAMS = [
         "required": True,
         "update_policy": UpdatePolicy.UNSUPPORTED
     }),
-    # Master
+    # Head node
     ("master_instance_type", {
-        "default": "t2.micro",
+        "type": HeadNodeInstanceTypeCfnParam,
         "cfn_param_mapping": "MasterInstanceType",
         "validators": [head_node_instance_type_validator, ec2_instance_type_validator],
         "update_policy": UpdatePolicy.UNSUPPORTED,
     }),
     ("master_root_volume_size", {
         "type": IntCfnParam,
-        "default": 25,
-        "allowed_values": ALLOWED_VALUES["greater_than_25"],
+        "default": 35,
+        "allowed_values": ALLOWED_VALUES["greater_than_35"],
         "cfn_param_mapping": "MasterRootVolumeSize",
         "update_policy": UpdatePolicy(
             UpdatePolicy.UNSUPPORTED,
@@ -787,11 +833,19 @@ CLUSTER_COMMON_PARAMS = [
             action_needed=UpdatePolicy.ACTIONS_NEEDED["ebs_volume_update"]
         )
     }),
+    ("base_os", {
+        "type": BaseOSCfnParam,
+        "cfn_param_mapping": "BaseOS",
+        "allowed_values": SUPPORTED_OSS,
+        "validators": [architecture_os_validator],
+        "required": True,
+        "update_policy": UpdatePolicy.UNSUPPORTED
+    }),
     # Compute fleet
     ("compute_root_volume_size", {
         "type": IntCfnParam,
-        "default": 25,
-        "allowed_values": ALLOWED_VALUES["greater_than_25"],
+        "default": 35,
+        "allowed_values": ALLOWED_VALUES["greater_than_35"],
         "cfn_param_mapping": "ComputeRootVolumeSize",
         "update_policy": UpdatePolicy.COMPUTE_FLEET_STOP
     }),
@@ -830,7 +884,7 @@ CLUSTER_COMMON_PARAMS = [
     ("enable_efa", {
         "allowed_values": ["compute"],
         "cfn_param_mapping": "EFA",
-        "validators": [efa_validator],
+        "validators": [efa_validator, efa_os_arch_validator],
         "update_policy": UpdatePolicy.UNSUPPORTED
     }),
     ("enable_efa_gdr", {
@@ -921,6 +975,7 @@ CLUSTER_COMMON_PARAMS = [
     }),
     ("vpc_settings", {
         "type": SettingsCfnParam,
+        "required": True,
         "referred_section": VPC,
         "update_policy": UpdatePolicy.UNSUPPORTED,
     }),
@@ -998,9 +1053,19 @@ CLUSTER_COMMON_PARAMS = [
     }),
     ("cluster_resource_bucket", {
         "cfn_param_mapping": "ResourcesS3Bucket",
-        "validators": [s3_bucket_validator],
+        "validators": [s3_bucket_validator, s3_bucket_region_validator],
         "update_policy": UpdatePolicy.READ_ONLY_RESOURCE_BUCKET,
     }),
+    ("iam_lambda_role", {
+        "cfn_param_mapping": "IAMLambdaRoleName",
+        "update_policy": UpdatePolicy.SUPPORTED,
+    }),
+    ("instance_types_data", {
+        "type": JsonCfnParam,
+        "default": {},
+        "cfn_param_mapping": "InstanceTypesData",
+        "update_policy": UpdatePolicy.UNSUPPORTED
+    }),
 ]
 
 
@@ -1025,9 +1090,7 @@ CLUSTER_SIT = {
             }),
             # Compute fleet
             ("compute_instance_type", {
-                "default":
-                    lambda section:
-                    "optimal" if section and section.get_param_value("scheduler") == "awsbatch" else "t2.micro",
+                "type": ComputeInstanceTypeCfnParam,
                 "cfn_param_mapping": "ComputeInstanceType",
                 "validators": [compute_instance_type_validator, instances_architecture_compatibility_validator],
                 "update_policy": UpdatePolicy.COMPUTE_FLEET_STOP
@@ -1042,7 +1105,7 @@ CLUSTER_SIT = {
                 "type": QueueSizeCfnParam,
                 "default": 10,
                 "cfn_param_mapping": "MaxSize",
-                "update_policy": UpdatePolicy.SUPPORTED
+                "update_policy": UpdatePolicy.MAX_QUEUE_SIZE
             }),
             ("maintain_initial_size", {
                 "type": MaintainInitialSizeCfnParam,
@@ -1074,6 +1137,7 @@ CLUSTER_SIT = {
                 "default": "ondemand",
                 "allowed_values": ["ondemand", "spot"],
                 "cfn_param_mapping": "ClusterType",
+                "validators": [cluster_type_validator],
                 "update_policy": UpdatePolicy.COMPUTE_FLEET_STOP
             }),
             ("spot_price", {
